@@ -77,20 +77,51 @@
     of the abstract types defined below.
 *)
 
+(* The V>=4.2 lines are not necessary for typing,
+   but they are necessary for the compatibility test in batteries_compattest.ml
+   which are of the form:
+     module _ = (BatBigarray : module type of Bigarray)
+   because of the somewhat strange interpretation of strengthening in (module type of),
+   we need to explicitly equate each type with its constructor *)
 type float32_elt = Bigarray.float32_elt
+ = Float32_elt
 type float64_elt = Bigarray.float64_elt
+ = Float64_elt
 type complex32_elt = Bigarray.complex32_elt
+ = Complex32_elt
 type complex64_elt = Bigarray.complex64_elt
+ = Complex64_elt
 type int8_signed_elt = Bigarray.int8_signed_elt
+ = Int8_signed_elt
 type int8_unsigned_elt = Bigarray.int8_unsigned_elt
+ = Int8_unsigned_elt
 type int16_signed_elt = Bigarray.int16_signed_elt
+ = Int16_signed_elt
 type int16_unsigned_elt = Bigarray.int16_unsigned_elt
+ = Int16_unsigned_elt
 type int_elt = Bigarray.int_elt
+ = Int_elt
 type int32_elt = Bigarray.int32_elt
+ = Int32_elt
 type int64_elt = Bigarray.int64_elt
+ = Int64_elt
 type nativeint_elt = Bigarray.nativeint_elt
+ = Nativeint_elt
 
 type ('a, 'b) kind = ('a,'b) Bigarray.kind
+           = Float32 : (float, float32_elt) kind
+           | Float64 : (float, float64_elt) kind
+           | Int8_signed : (int, int8_signed_elt) kind
+           | Int8_unsigned : (int, int8_unsigned_elt) kind
+           | Int16_signed : (int, int16_signed_elt) kind
+           | Int16_unsigned : (int, int16_unsigned_elt) kind
+           | Int32 : (int32, int32_elt) kind
+           | Int64 : (int64, int64_elt) kind
+           | Int : (int, int_elt) kind
+           | Nativeint : (nativeint, nativeint_elt) kind
+           | Complex32 : (Complex.t, complex32_elt) kind
+           | Complex64 : (Complex.t, complex64_elt) kind
+           | Char : (char, int8_unsigned_elt) kind
 (** To each element kind is associated an OCaml type, which is
     the type of OCaml values that can be stored in the big array
     or read back from it.  This type is not necessarily the same
@@ -100,12 +131,34 @@ type ('a, 'b) kind = ('a,'b) Bigarray.kind
     its elements from OCaml uses the OCaml type [float], which is
     64-bit double precision floats.
 
-    The abstract type [('a, 'b) kind] captures this association
-    of an OCaml type ['a] for values read or written in the big array,
-    and of an element kind ['b] which represents the actual contents
-    of the big array.  The following predefined values of type
-    [kind] list all possible associations of OCaml types with
-    element kinds: *)
+
+
+
+
+
+
+   The GADT type [('a, 'b) kind] captures this association
+   of an OCaml type ['a] for values read or written in the big array,
+   and of an element kind ['b] which represents the actual contents
+   of the big array. Its constructors list all possible associations
+   of OCaml types with element kinds, and are re-exported below for
+   backward-compatibility reasons.
+
+   Using a generalized algebraic datatype (GADT) here allows to write
+   well-typed polymorphic functions whose return type depend on the
+   argument type, such as:
+{[
+  let zero : type a b. (a, b) kind -> a = function
+    | Float32 -> 0.0 | Complex32 -> Complex.zero
+    | Float64 -> 0.0 | Complex64 -> Complex.zero
+    | Int8_signed -> 0 | Int8_unsigned -> 0
+    | Int16_signed -> 0 | Int16_unsigned -> 0
+    | Int32 -> 0l | Int64 -> 0L
+    | Int -> 0 | Nativeint -> 0n
+    | Char -> '\000'
+]}
+*)
+
 
 val float32 : (float, float32_elt) kind
 (** See {!Bigarray.char}. *)
@@ -162,9 +215,11 @@ val char : (char, int8_unsigned_elt) kind
 (** {6 Array layouts} *)
 
 type c_layout = Bigarray.c_layout
+ = C_layout_typ
 (** See {!Bigarray.fortran_layout}.*)
 
 type fortran_layout = Bigarray.fortran_layout
+ = Fortran_layout_typ
 (** To facilitate interoperability with existing C and Fortran code,
     this library supports two different memory layouts for big arrays,
     one compatible with the C conventions,
@@ -188,6 +243,8 @@ type fortran_layout = Bigarray.fortran_layout
     abstract types {!Bigarray.c_layout} and [fortran_layout] respectively. *)
 
 type 'a layout = 'a Bigarray.layout
+           = C_layout : c_layout layout
+           | Fortran_layout : fortran_layout layout
 (** The type ['a layout] represents one of the two supported
     memory layouts: C-style if ['a] is {!Bigarray.c_layout}, Fortran-style
     if ['a] is {!Bigarray.fortran_layout}. *)
@@ -210,61 +267,61 @@ module Genarray :
 sig
   type ('a, 'b, 'c) t = ('a, 'b, 'c) Bigarray.Genarray.t
   (** The type [Genarray.t] is the type of big arrays with variable
-	  numbers of dimensions.  Any number of dimensions between 1 and 16
-	  is supported.
+      numbers of dimensions.  Any number of dimensions between 1 and 16
+      is supported.
 
-	  The three type parameters to [Genarray.t] identify the array element
-	  kind and layout, as follows:
-	  - the first parameter, ['a], is the OCaml type for accessing array
-	  elements ([float], [int], [int32], [int64], [nativeint]);
-	  - the second parameter, ['b], is the actual kind of array elements
-	  ([float32_elt], [float64_elt], [int8_signed_elt], [int8_unsigned_elt],
-	  etc);
-	  - the third parameter, ['c], identifies the array layout
-	  ([c_layout] or [fortran_layout]).
+      The three type parameters to [Genarray.t] identify the array element
+      kind and layout, as follows:
+      - the first parameter, ['a], is the OCaml type for accessing array
+      elements ([float], [int], [int32], [int64], [nativeint]);
+      - the second parameter, ['b], is the actual kind of array elements
+      ([float32_elt], [float64_elt], [int8_signed_elt], [int8_unsigned_elt],
+      etc);
+      - the third parameter, ['c], identifies the array layout
+      ([c_layout] or [fortran_layout]).
 
-	  For instance, [(float, float32_elt, fortran_layout) Genarray.t]
-	  is the type of generic big arrays containing 32-bit floats
-	  in Fortran layout; reads and writes in this array use the
-	  OCaml type [float]. *)
+      For instance, [(float, float32_elt, fortran_layout) Genarray.t]
+      is the type of generic big arrays containing 32-bit floats
+      in Fortran layout; reads and writes in this array use the
+      OCaml type [float]. *)
 
   external create: ('a, 'b) kind -> 'c layout -> int array -> ('a, 'b, 'c) t
     = "caml_ba_create"
   (** [Genarray.create kind layout dimensions] returns a new big array
-	  whose element kind is determined by the parameter [kind] (one of
-	  [float32], [float64], [int8_signed], etc) and whose layout is
-	  determined by the parameter [layout] (one of [c_layout] or
-	  [fortran_layout]).  The [dimensions] parameter is an array of
-	  integers that indicate the size of the big array in each dimension.
-	  The length of [dimensions] determines the number of dimensions
-	  of the bigarray.
+      whose element kind is determined by the parameter [kind] (one of
+      [float32], [float64], [int8_signed], etc) and whose layout is
+      determined by the parameter [layout] (one of [c_layout] or
+      [fortran_layout]).  The [dimensions] parameter is an array of
+      integers that indicate the size of the big array in each dimension.
+      The length of [dimensions] determines the number of dimensions
+      of the bigarray.
 
-	  For instance, [Genarray.create int32 c_layout [|4;6;8|]]
-	  returns a fresh big array of 32-bit integers, in C layout,
-	  having three dimensions, the three dimensions being 4, 6 and 8
-	  respectively.
+      For instance, [Genarray.create int32 c_layout [|4;6;8|]]
+      returns a fresh big array of 32-bit integers, in C layout,
+      having three dimensions, the three dimensions being 4, 6 and 8
+      respectively.
 
-	  Big arrays returned by [Genarray.create] are not initialized:
-	  the initial values of array elements is unspecified.
+      Big arrays returned by [Genarray.create] are not initialized:
+      the initial values of array elements is unspecified.
 
-	  @raise Invalid_argument if the number of dimensions
-	  is not in the range 1 to 16 inclusive, or if one of the dimensions
-	  is negative. *)
+      @raise Invalid_argument if the number of dimensions
+      is not in the range 1 to 16 inclusive, or if one of the dimensions
+      is negative. *)
 
   external num_dims: ('a, 'b, 'c) t -> int = "caml_ba_num_dims"
   (** Return the number of dimensions of the given big array. *)
 
   val dims : ('a, 'b, 'c) t -> int array
   (** [Genarray.dims a] returns all dimensions of the big array [a],
-	as an array of integers of length [Genarray.num_dims a]. *)
+    as an array of integers of length [Genarray.num_dims a]. *)
 
   external nth_dim: ('a, 'b, 'c) t -> int -> int = "caml_ba_dim"
   (** [Genarray.nth_dim a n] returns the [n]-th dimension of the
-	big array [a].  The first dimension corresponds to [n = 0];
-	the second dimension corresponds to [n = 1]; the last dimension,
-	to [n = Genarray.num_dims a - 1].
-	@raise Invalid_argument if [n] is less than 0 or greater or equal than
-	[Genarray.num_dims a]. *)
+    big array [a].  The first dimension corresponds to [n = 0];
+    the second dimension corresponds to [n = 1]; the last dimension,
+    to [n = Genarray.num_dims a - 1].
+    @raise Invalid_argument if [n] is less than 0 or greater or equal than
+    [Genarray.num_dims a]. *)
 
   external kind: ('a, 'b, 'c) t -> ('a, 'b) kind = "caml_ba_kind"
   (** Return the kind of the given big array. *)
@@ -274,175 +331,175 @@ sig
 
   external get: ('a, 'b, 'c) t -> int array -> 'a = "caml_ba_get_generic"
   (** Read an element of a generic big array.
-	[Genarray.get a [|i1; ...; iN|]] returns the element of [a]
-	whose coordinates are [i1] in the first dimension, [i2] in
-	the second dimension, ..., [iN] in the [N]-th dimension.
+    [Genarray.get a [|i1; ...; iN|]] returns the element of [a]
+    whose coordinates are [i1] in the first dimension, [i2] in
+    the second dimension, ..., [iN] in the [N]-th dimension.
 
-	If [a] has C layout, the coordinates must be greater or equal than 0
-	and strictly less than the corresponding dimensions of [a].
-	If [a] has Fortran layout, the coordinates must be greater or equal
-	than 1 and less or equal than the corresponding dimensions of [a].
-	@raise Invalid_argument if the array [a] does not have exactly [N]
-	dimensions, or if the coordinates are outside the array bounds.
+    If [a] has C layout, the coordinates must be greater or equal than 0
+    and strictly less than the corresponding dimensions of [a].
+    If [a] has Fortran layout, the coordinates must be greater or equal
+    than 1 and less or equal than the corresponding dimensions of [a].
+    @raise Invalid_argument if the array [a] does not have exactly [N]
+    dimensions, or if the coordinates are outside the array bounds.
 
-	If [N > 3], alternate syntax is provided: you can write
-	[a.{i1, i2, ..., iN}] instead of [Genarray.get a [|i1; ...; iN|]].
-	(The syntax [a.{...}] with one, two or three coordinates is
-	reserved for accessing one-, two- and three-dimensional arrays
-	as described below.) *)
+    If [N > 3], alternate syntax is provided: you can write
+    [a.{i1, i2, ..., iN}] instead of [Genarray.get a [|i1; ...; iN|]].
+    (The syntax [a.{...}] with one, two or three coordinates is
+    reserved for accessing one-, two- and three-dimensional arrays
+    as described below.) *)
 
   external set: ('a, 'b, 'c) t -> int array -> 'a -> unit
     = "caml_ba_set_generic"
   (** Assign an element of a generic big array.
-	[Genarray.set a [|i1; ...; iN|] v] stores the value [v] in the
-	element of [a] whose coordinates are [i1] in the first dimension,
-	[i2] in the second dimension, ..., [iN] in the [N]-th dimension.
+    [Genarray.set a [|i1; ...; iN|] v] stores the value [v] in the
+    element of [a] whose coordinates are [i1] in the first dimension,
+    [i2] in the second dimension, ..., [iN] in the [N]-th dimension.
 
-	The array [a] must have exactly [N] dimensions, and all coordinates
-	must lie inside the array bounds, as described for [Genarray.get];
-	@raise Invalid_argument otherwise.
+    The array [a] must have exactly [N] dimensions, and all coordinates
+    must lie inside the array bounds, as described for [Genarray.get];
+    @raise Invalid_argument otherwise.
 
-	If [N > 3], alternate syntax is provided: you can write
-	[a.{i1, i2, ..., iN} <- v] instead of
-	[Genarray.set a [|i1; ...; iN|] v].
-	(The syntax [a.{...} <- v] with one, two or three coordinates is
-	reserved for updating one-, two- and three-dimensional arrays
-	as described below.) *)
+    If [N > 3], alternate syntax is provided: you can write
+    [a.{i1, i2, ..., iN} <- v] instead of
+    [Genarray.set a [|i1; ...; iN|] v].
+    (The syntax [a.{...} <- v] with one, two or three coordinates is
+    reserved for updating one-, two- and three-dimensional arrays
+    as described below.) *)
 
   external sub_left: ('a, 'b, c_layout) t -> int -> int -> ('a, 'b, c_layout) t
     = "caml_ba_sub"
   (** Extract a sub-array of the given big array by restricting the
-	first (left-most) dimension.  [Genarray.sub_left a ofs len]
-	returns a big array with the same number of dimensions as [a],
-	and the same dimensions as [a], except the first dimension,
-	which corresponds to the interval [[ofs ... ofs + len - 1]]
-	of the first dimension of [a].  No copying of elements is
-	involved: the sub-array and the original array share the same
-	storage space.  In other terms, the element at coordinates
-	[[|i1; ...; iN|]] of the sub-array is identical to the
-	element at coordinates [[|i1+ofs; ...; iN|]] of the original
-	array [a].
+    first (left-most) dimension.  [Genarray.sub_left a ofs len]
+    returns a big array with the same number of dimensions as [a],
+    and the same dimensions as [a], except the first dimension,
+    which corresponds to the interval [[ofs ... ofs + len - 1]]
+    of the first dimension of [a].  No copying of elements is
+    involved: the sub-array and the original array share the same
+    storage space.  In other terms, the element at coordinates
+    [[|i1; ...; iN|]] of the sub-array is identical to the
+    element at coordinates [[|i1+ofs; ...; iN|]] of the original
+    array [a].
 
-	[Genarray.sub_left] applies only to big arrays in C layout.
-	@raise Invalid_argument if [ofs] and [len] do not designate
-	a valid sub-array of [a], that is, if [ofs < 0], or [len < 0],
-	or [ofs + len > Genarray.nth_dim a 0]. *)
+    [Genarray.sub_left] applies only to big arrays in C layout.
+    @raise Invalid_argument if [ofs] and [len] do not designate
+    a valid sub-array of [a], that is, if [ofs < 0], or [len < 0],
+    or [ofs + len > Genarray.nth_dim a 0]. *)
 
   external sub_right:
     ('a, 'b, fortran_layout) t -> int -> int -> ('a, 'b, fortran_layout) t
     = "caml_ba_sub"
   (** Extract a sub-array of the given big array by restricting the
-	last (right-most) dimension.  [Genarray.sub_right a ofs len]
-	returns a big array with the same number of dimensions as [a],
-	and the same dimensions as [a], except the last dimension,
-	which corresponds to the interval [[ofs ... ofs + len - 1]]
-	of the last dimension of [a].  No copying of elements is
-	involved: the sub-array and the original array share the same
-	storage space.  In other terms, the element at coordinates
-	[[|i1; ...; iN|]] of the sub-array is identical to the
-	element at coordinates [[|i1; ...; iN+ofs|]] of the original
-	array [a].
+    last (right-most) dimension.  [Genarray.sub_right a ofs len]
+    returns a big array with the same number of dimensions as [a],
+    and the same dimensions as [a], except the last dimension,
+    which corresponds to the interval [[ofs ... ofs + len - 1]]
+    of the last dimension of [a].  No copying of elements is
+    involved: the sub-array and the original array share the same
+    storage space.  In other terms, the element at coordinates
+    [[|i1; ...; iN|]] of the sub-array is identical to the
+    element at coordinates [[|i1; ...; iN+ofs|]] of the original
+    array [a].
 
-	[Genarray.sub_right] applies only to big arrays in Fortran layout.
-	@raise Invalid_argument if [ofs] and [len] do not designate
-	a valid sub-array of [a], that is, if [ofs < 1], or [len < 0],
-	or [ofs + len > Genarray.nth_dim a (Genarray.num_dims a - 1)]. *)
+    [Genarray.sub_right] applies only to big arrays in Fortran layout.
+    @raise Invalid_argument if [ofs] and [len] do not designate
+    a valid sub-array of [a], that is, if [ofs < 1], or [len < 0],
+    or [ofs + len > Genarray.nth_dim a (Genarray.num_dims a - 1)]. *)
 
   external slice_left:
     ('a, 'b, c_layout) t -> int array -> ('a, 'b, c_layout) t
     = "caml_ba_slice"
   (** Extract a sub-array of lower dimension from the given big array
-	by fixing one or several of the first (left-most) coordinates.
-	[Genarray.slice_left a [|i1; ... ; iM|]] returns the ``slice''
-	of [a] obtained by setting the first [M] coordinates to
-	[i1], ..., [iM].  If [a] has [N] dimensions, the slice has
-	dimension [N - M], and the element at coordinates
-	[[|j1; ...; j(N-M)|]] in the slice is identical to the element
-	at coordinates [[|i1; ...; iM; j1; ...; j(N-M)|]] in the original
-	array [a].  No copying of elements is involved: the slice and
-	the original array share the same storage space.
+    by fixing one or several of the first (left-most) coordinates.
+    [Genarray.slice_left a [|i1; ... ; iM|]] returns the ``slice''
+    of [a] obtained by setting the first [M] coordinates to
+    [i1], ..., [iM].  If [a] has [N] dimensions, the slice has
+    dimension [N - M], and the element at coordinates
+    [[|j1; ...; j(N-M)|]] in the slice is identical to the element
+    at coordinates [[|i1; ...; iM; j1; ...; j(N-M)|]] in the original
+    array [a].  No copying of elements is involved: the slice and
+    the original array share the same storage space.
 
-	[Genarray.slice_left] applies only to big arrays in C layout.
-	@raise Invalid_argument if [M >= N], or if [[|i1; ... ; iM|]]
-	is outside the bounds of [a]. *)
+    [Genarray.slice_left] applies only to big arrays in C layout.
+    @raise Invalid_argument if [M >= N], or if [[|i1; ... ; iM|]]
+    is outside the bounds of [a]. *)
 
   external slice_right:
     ('a, 'b, fortran_layout) t -> int array -> ('a, 'b, fortran_layout) t
     = "caml_ba_slice"
   (** Extract a sub-array of lower dimension from the given big array
-	by fixing one or several of the last (right-most) coordinates.
-	[Genarray.slice_right a [|i1; ... ; iM|]] returns the ``slice''
-	of [a] obtained by setting the last [M] coordinates to
-	[i1], ..., [iM].  If [a] has [N] dimensions, the slice has
-	dimension [N - M], and the element at coordinates
-	[[|j1; ...; j(N-M)|]] in the slice is identical to the element
-	at coordinates [[|j1; ...; j(N-M); i1; ...; iM|]] in the original
-	array [a].  No copying of elements is involved: the slice and
-	the original array share the same storage space.
+    by fixing one or several of the last (right-most) coordinates.
+    [Genarray.slice_right a [|i1; ... ; iM|]] returns the ``slice''
+    of [a] obtained by setting the last [M] coordinates to
+    [i1], ..., [iM].  If [a] has [N] dimensions, the slice has
+    dimension [N - M], and the element at coordinates
+    [[|j1; ...; j(N-M)|]] in the slice is identical to the element
+    at coordinates [[|j1; ...; j(N-M); i1; ...; iM|]] in the original
+    array [a].  No copying of elements is involved: the slice and
+    the original array share the same storage space.
 
-	[Genarray.slice_right] applies only to big arrays in Fortran layout.
-	@raise Invalid_argument if [M >= N], or if [[|i1; ... ; iM|]]
-	is outside the bounds of [a]. *)
+    [Genarray.slice_right] applies only to big arrays in Fortran layout.
+    @raise Invalid_argument if [M >= N], or if [[|i1; ... ; iM|]]
+    is outside the bounds of [a]. *)
 
   external blit: ('a, 'b, 'c) t -> ('a, 'b, 'c) t -> unit
     = "caml_ba_blit"
   (** Copy all elements of a big array in another big array.
-	[Genarray.blit src dst] copies all elements of [src] into
-	[dst].  Both arrays [src] and [dst] must have the same number of
-	dimensions and equal dimensions.  Copying a sub-array of [src]
-	to a sub-array of [dst] can be achieved by applying [Genarray.blit]
-	to sub-array or slices of [src] and [dst]. *)
+    [Genarray.blit src dst] copies all elements of [src] into
+    [dst].  Both arrays [src] and [dst] must have the same number of
+    dimensions and equal dimensions.  Copying a sub-array of [src]
+    to a sub-array of [dst] can be achieved by applying [Genarray.blit]
+    to sub-array or slices of [src] and [dst]. *)
 
   external fill: ('a, 'b, 'c) t -> 'a -> unit = "caml_ba_fill"
   (** Set all elements of a big array to a given value.
-	[Genarray.fill a v] stores the value [v] in all elements of
-	the big array [a].  Setting only some elements of [a] to [v]
-	can be achieved by applying [Genarray.fill] to a sub-array
-	or a slice of [a]. *)
+    [Genarray.fill a v] stores the value [v] in all elements of
+    the big array [a].  Setting only some elements of [a] to [v]
+    can be achieved by applying [Genarray.fill] to a sub-array
+    or a slice of [a]. *)
 
   val map_file:
     Unix.file_descr -> ?pos:int64 -> ('a, 'b) kind -> 'c layout ->
     bool -> int array -> ('a, 'b, 'c) t
   (** Memory mapping of a file as a big array.
-	[Genarray.map_file fd kind layout shared dims]
-	returns a big array of kind [kind], layout [layout],
-	and dimensions as specified in [dims].  The data contained in
-	this big array are the contents of the file referred to by
-	the file descriptor [fd] (as opened previously with
-	[Unix.openfile], for example).  The optional [pos] parameter
-	is the byte offset in the file of the data being mapped;
-	it default to 0 (map from the beginning of the file).
+    [Genarray.map_file fd kind layout shared dims]
+    returns a big array of kind [kind], layout [layout],
+    and dimensions as specified in [dims].  The data contained in
+    this big array are the contents of the file referred to by
+    the file descriptor [fd] (as opened previously with
+    [Unix.openfile], for example).  The optional [pos] parameter
+    is the byte offset in the file of the data being mapped;
+    it default to 0 (map from the beginning of the file).
 
-	If [shared] is [true], all modifications performed on the array
-	are reflected in the file.  This requires that [fd] be opened
-	with write permissions.  If [shared] is [false], modifications
-	performed on the array are done in memory only, using
-	copy-on-write of the modified pages; the underlying file is not
-	affected.
+    If [shared] is [true], all modifications performed on the array
+    are reflected in the file.  This requires that [fd] be opened
+    with write permissions.  If [shared] is [false], modifications
+    performed on the array are done in memory only, using
+    copy-on-write of the modified pages; the underlying file is not
+    affected.
 
-	[Genarray.map_file] is much more efficient than reading
-	the whole file in a big array, modifying that big array,
-	and writing it afterwards.
+    [Genarray.map_file] is much more efficient than reading
+    the whole file in a big array, modifying that big array,
+    and writing it afterwards.
 
-	To adjust automatically the dimensions of the big array to
-	the actual size of the file, the major dimension (that is,
-	the first dimension for an array with C layout, and the last
-	dimension for an array with Fortran layout) can be given as
-	[-1].  [Genarray.map_file] then determines the major dimension
-	from the size of the file.  The file must contain an integral
-	number of sub-arrays as determined by the non-major dimensions,
-	@raise Failure otherwise.
+    To adjust automatically the dimensions of the big array to
+    the actual size of the file, the major dimension (that is,
+    the first dimension for an array with C layout, and the last
+    dimension for an array with Fortran layout) can be given as
+    [-1].  [Genarray.map_file] then determines the major dimension
+    from the size of the file.  The file must contain an integral
+    number of sub-arrays as determined by the non-major dimensions,
+    @raise Failure otherwise.
 
-	If all dimensions of the big array are given, the file size is
-	matched against the size of the big array.  If the file is larger
-	than the big array, only the initial portion of the file is
-	mapped to the big array.  If the file is smaller than the big
-	array, the file is automatically grown to the size of the big array.
-	This requires write permissions on [fd]. *)
+    If all dimensions of the big array are given, the file size is
+    matched against the size of the big array.  If the file is larger
+    than the big array, only the initial portion of the file is
+    mapped to the big array.  If the file is smaller than the big
+    array, the file is automatically grown to the size of the big array.
+    This requires write permissions on [fd]. *)
 
   val iter : ('a -> unit) -> ('a, 'b, 'c) t -> unit
   (** [iter f a] applies function [f] in turn to all
-	the elements of [a].  *)
+    the elements of [a].  *)
 
   val iteri : ((int, [`Read]) BatArray.Cap.t -> 'a -> unit) -> ('a, 'b, 'c) t -> unit
   (** Same as {!iter}, but the function is applied to the index of
@@ -460,7 +517,7 @@ sig
 
   val enum : ('a, 'b, 'c) t -> 'a BatEnum.t
   (** [enum e] returns an enumeration on the elements of [e].
-	The order of enumeration is unspecified.*)
+    The order of enumeration is unspecified.*)
 
   val map :
     ('a -> 'b) ->
@@ -499,7 +556,7 @@ module Array1 : sig
       determine the array element kind and the array layout
       as described for [Genarray.create]. *)
 
-  val dim: ('a, 'b, 'c) t -> int
+
   external dim: ('a, 'b, 'c) t -> int = "%caml_ba_dim_1"
   (** Return the size (dimension) of the given one-dimensional
       big array. *)
@@ -573,8 +630,8 @@ module Array1 : sig
     (int -> 'a -> 'b) ->
     ('b, 'c) Bigarray.kind -> ('a, 'd, 'e) t -> ('b, 'c, 'e) t
   (** Same as {!Bigarray.Array1.map}, but the
-	function is applied to the index of the element as the first argument,
-	and the element itself as the second argument. *)
+    function is applied to the index of the element as the first argument,
+    and the element itself as the second argument. *)
 
   val modify : ('a -> 'a) -> ('a, 'b, 'c) t -> unit
   (** [modify f a] changes each element [x] in [a] to [f x]
@@ -587,7 +644,7 @@ module Array1 : sig
 
   val to_array : ('a, 'b, 'c) t -> 'a array
   (** Build a one-dimensional array initialized from the
-	given big array.  *)
+    given big array.  *)
 
 
   (**{6 Unsafe operations}
@@ -627,11 +684,11 @@ sig
       determine the array element kind and the array layout
       as described for {!Bigarray.Genarray.create}. *)
 
-  val dim1: ('a, 'b, 'c) t -> int
+
   external dim1: ('a, 'b, 'c) t -> int = "%caml_ba_dim_1"
   (** Return the first dimension of the given two-dimensional big array. *)
 
-  val dim2: ('a, 'b, 'c) t -> int
+
   external dim2: ('a, 'b, 'c) t -> int = "%caml_ba_dim_2"
   (** Return the second dimension of the given two-dimensional big array. *)
 
@@ -705,7 +762,7 @@ sig
 
   val enum : ('a, 'b, 'c) t -> 'a BatEnum.t
   (** [enum e] returns an enumeration on the elements of [e].
-	The order of enumeration is unspecified.*)
+    The order of enumeration is unspecified.*)
 
   val map :
     ('a -> 'b) ->
@@ -768,15 +825,15 @@ sig
       [kind] and [layout] determine the array element kind and
       the array layout as described for {!Bigarray.Genarray.create}. *)
 
-  val dim1: ('a, 'b, 'c) t -> int
+
   external dim1: ('a, 'b, 'c) t -> int = "%caml_ba_dim_1"
   (** Return the first dimension of the given three-dimensional big array. *)
 
-  val dim2: ('a, 'b, 'c) t -> int
+
   external dim2: ('a, 'b, 'c) t -> int = "%caml_ba_dim_2"
   (** Return the second dimension of the given three-dimensional big array. *)
 
-  val dim3: ('a, 'b, 'c) t -> int
+
   external dim3: ('a, 'b, 'c) t -> int = "%caml_ba_dim_3"
   (** Return the third dimension of the given three-dimensional big array. *)
 
@@ -871,7 +928,7 @@ sig
 
   val enum : ('a, 'b, 'c) t -> 'a BatEnum.t
   (** [enum e] returns an enumeration on the elements of [e].
-	The order of enumeration is unspecified.*)
+    The order of enumeration is unspecified.*)
 
   val map :
     ('a -> 'b) ->
