@@ -88,21 +88,29 @@ val output: _ BatInnerIO.output -> ?sharing:bool -> ?closures:bool -> 'a -> unit
     un-marshaling time, using an MD5 digest of the code transmitted
     along with the code position.) *)
 
+external to_bytes :
+  'a -> extern_flags list -> Bytes.t = "caml_output_value_to_string"
+(** [Marshal.to_bytes v flags] returns a byte sequence containing
+   the representation of [v].
+   The [flags] argument has the same meaning as for
+   {!Marshal.output}.
+
+   @since 2.3.0
+*)
+
 external to_string :
   'a -> extern_flags list -> string = "caml_output_value_to_string"
-(** [Marshal.to_string v flags] returns a string containing
-    the representation of [v] as a sequence of bytes.
-    The [flags] argument has the same meaning as for
-    {!Marshal.to_channel}. *)
+(** Same as [to_bytes] but return the result as a string instead of
+    a byte sequence. *)
 
-val to_buffer : string -> int -> int -> 'a -> extern_flags list -> int
+val to_buffer : Bytes.t -> int -> int -> 'a -> extern_flags list -> int
 (** [Marshal.to_buffer buff ofs len v flags] marshals the value [v],
-    storing its byte representation in the string [buff],
-    starting at character number [ofs], and writing at most
-    [len] characters.  It returns the number of characters
-    actually written to the string. If the byte representation
-    of [v] does not fit in [len] characters, the exception [Failure]
-    is raised. *)
+   storing its byte representation in the sequence [buff],
+   starting at index [ofs], and writing at most
+   [len] bytes.  It returns the number of bytes
+   actually written to the sequence. If the byte representation
+   of [v] does not fit in [len] characters, the exception [Failure]
+   is raised. *)
 
 val input : BatInnerIO.input -> 'a
 (** [input inp] reads from [inp] the
@@ -110,38 +118,46 @@ val input : BatInnerIO.input -> 'a
     one of the [Marshal.to_*] functions, and reconstructs and
     returns the corresponding value.*)
 
+val from_bytes : Bytes.t -> int -> 'a
+(** [Marshal.from_bytes buff ofs] unmarshals a structured value
+   like {!Marshal.from_channel} does, except that the byte
+   representation is not read from a channel, but taken from
+   the byte sequence [buff], starting at position [ofs].
+   The byte sequence is not mutated.
+   
+   @since 2.3.0
+*)
+
 val from_string : string -> int -> 'a
-(** [Marshal.from_string buff ofs] unmarshals a structured value
-    like {!Marshal.from_channel} does, except that the byte
-    representation is not read from a channel, but taken from
-    the string [buff], starting at position [ofs]. *)
+(** Same as [from_bytes] but take a string as argument instead of a
+    byte sequence. *)
 
 val header_size : int
 (** The bytes representing a marshaled value are composed of
-    a fixed-size header and a variable-sized data part,
-    whose size can be determined from the header.
-    {!Marshal.header_size} is the size, in characters, of the header.
-    {!Marshal.data_size}[ buff ofs] is the size, in characters,
-    of the data part, assuming a valid header is stored in
-    [buff] starting at position [ofs].
-    Finally, {!Marshal.total_size}[ buff ofs] is the total size,
-    in characters, of the marshaled value.
-    Both {!Marshal.data_size} and {!Marshal.total_size} raise [Failure]
-    if [buff], [ofs] does not contain a valid header.
+   a fixed-size header and a variable-sized data part,
+   whose size can be determined from the header.
+   {!Marshal.header_size} is the size, in bytes, of the header.
+   {!Marshal.data_size}[ buff ofs] is the size, in bytes,
+   of the data part, assuming a valid header is stored in
+   [buff] starting at position [ofs].
+   Finally, {!Marshal.total_size} [buff ofs] is the total size,
+   in bytes, of the marshaled value.
+   Both {!Marshal.data_size} and {!Marshal.total_size} raise [Failure]
+   if [buff], [ofs] does not contain a valid header.
 
-    To read the byte representation of a marshaled value into
-    a string buffer, the program needs to read first
-    {!Marshal.header_size} characters into the buffer,
-    then determine the length of the remainder of the
-    representation using {!Marshal.data_size},
-    make sure the buffer is large enough to hold the remaining
-    data, then read it, and finally call {!Marshal.from_string}
-    to unmarshal the value. *)
+   To read the byte representation of a marshaled value into
+   a byte sequence, the program needs to read first
+   {!Marshal.header_size} bytes into the sequence,
+   then determine the length of the remainder of the
+   representation using {!Marshal.data_size},
+   make sure the sequence is large enough to hold the remaining
+   data, then read it, and finally call {!Marshal.from_bytes}
+   to unmarshal the value. *)
 
-val data_size : string -> int -> int
+val data_size : Bytes.t -> int -> int
 (** See {!Marshal.header_size}.*)
 
-val total_size : string -> int -> int
+val total_size : Bytes.t -> int -> int
 (** See {!Marshal.header_size}.*)
 
 (** {6 Deprecated} *)
