@@ -280,23 +280,37 @@ val openfile : string -> open_flag list -> file_perm -> file_descr
 val close : file_descr -> unit
 (** Close a file descriptor. *)
 
-val read : file_descr -> string -> int -> int -> int
+val read : file_descr -> Bytes.t -> int -> int -> int
 (** [read fd buff ofs len] reads [len] characters from descriptor
     [fd], storing them in string [buff], starting at position [ofs]
     in string [buff]. Return the number of characters actually read. *)
 
-val write : file_descr -> string -> int -> int -> int
+val write : file_descr -> Bytes.t -> int -> int -> int
 (** [write fd buff ofs len] writes [len] characters to descriptor
     [fd], taking them from string [buff], starting at position [ofs]
     in string [buff]. Return the number of characters actually
     written.  [write] repeats the writing operation until all characters
     have been written or an error occurs.  *)
 
-val single_write : file_descr -> string -> int -> int -> int
+val single_write : file_descr -> Bytes.t -> int -> int -> int
 (** Same as [write], but attempts to write only once.
     Thus, if an error occurs, [single_write] guarantees that no data
     has been written. *)
 
+val write_substring : file_descr -> string -> int -> int -> int
+(** Same as [write], but take the data from a string instead of a byte
+    sequence.
+
+    @since 2.3.0
+*)
+
+val single_write_substring : file_descr -> string -> int -> int -> int
+(** Same as [single_write], but take the data from a string instead of
+    a byte sequence.
+
+    @since 2.3.0
+*)
+
 (** {6 Interfacing with the standard input/output library} *)
 
 open BatInnerIO
@@ -1133,20 +1147,35 @@ type msg_flag = Unix.msg_flag =
   (** The flags for {!Unix.recv},  {!Unix.recvfrom},
       {!Unix.send} and {!Unix.sendto}. *)
 
-val recv : file_descr -> string -> int -> int -> msg_flag list -> int
+val recv : file_descr -> Bytes.t -> int -> int -> msg_flag list -> int
 (** Receive data from a connected socket. *)
 
 val recvfrom :
-  file_descr -> string -> int -> int -> msg_flag list -> int * sockaddr
+  file_descr -> Bytes.t -> int -> int -> msg_flag list -> int * sockaddr
 (** Receive data from an unconnected socket. *)
 
-val send : file_descr -> string -> int -> int -> msg_flag list -> int
+val send : file_descr -> Bytes.t -> int -> int -> msg_flag list -> int
 (** Send data over a connected socket. *)
 
+val send_substring : file_descr -> string -> int -> int -> msg_flag list -> int
+(** Same as [send], but take the data from a string instead of a byte
+    sequence.
+
+    @since 2.3.0
+*)
+
 val sendto :
-  file_descr -> string -> int -> int -> msg_flag list -> sockaddr -> int
+  file_descr -> Bytes.t -> int -> int -> msg_flag list -> sockaddr -> int
 (** Send data over an unconnected socket. *)
 
+val sendto_substring :
+  file_descr -> string -> int -> int -> msg_flag list -> sockaddr -> int
+(** Same as [sendto], but take the data from a string instead of a
+    byte sequence.
+
+    @since 2.3.0
+*)
+
 
 
 (** {6 Socket options} *)
